@@ -2,6 +2,7 @@ import threading
 import shared
 import time
 import sys
+import os
 import pickle
 
 import tr#anslate
@@ -10,16 +11,16 @@ from debug import logger
 
 '''The singleCleaner class is a timer-driven thread that cleans data structures to free memory, resends messages when a remote node doesn't respond, and sends pong messages to keep connections alive if the network isn't busy.
 It cleans these data structures in memory:
-    inventory (moves data to the on-disk sql database)
-    inventorySets (clears then reloads data out of sql database)
+inventory (moves data to the on-disk sql database)
+inventorySets (clears then reloads data out of sql database)
 
 It cleans these tables on the disk:
-    inventory (clears data more than 2 days and 12 hours old)
-    pubkeys (clears pubkeys older than 4 weeks old which we have not used personally)
+inventory (clears data more than 2 days and 12 hours old)
+pubkeys (clears pubkeys older than 4 weeks old which we have not used personally)
 
 It resends messages when there has been no response:
-    resends getpubkey messages in 4 days (then 8 days, then 16 days, etc...)
-    resends msg messages in 4 days (then 8 days, then 16 days, etc...)
+resends getpubkey messages in 5 days (then 10 days, then 20 days, etc...)
+resends msg messages in 5 days (then 10 days, then 20 days, etc...)
 
 '''
 
@@ -31,11 +32,16 @@ class singleCleaner(threading.Thread):
 
     def run(self):
         timeWeLastClearedInventoryAndPubkeysTables = 0
+        try:
+            shared.maximumLengthOfTimeToBotherResendingMessages = (float(shared.config.get('bitmessagesettings', 'stopresendingafterxdays')) * 24 * 60 * 60) + (float(shared.config.get('bitmessagesettings', 'stopresendingafterxmonths')) * (60 * 60 * 24 *365)/12)
+        except:
+            # Either the user hasn't set stopresendingafterxdays and stopresendingafterxmonths yet or the options are missing from the config file.
+            shared.maximumLengthOfTimeToBotherResendingMessages = float('inf')
 
         while True:
             shared.UISignalQueue.put((
                 'updateStatusBar', 'Doing housekeeping (Flushing inventory in memory to disk...)'))
-            
+
             with shared.inventoryLock: # If you use both the inventoryLock and the sqlLock, always use the inventoryLock OUTSIDE of the sqlLock.
                 with SqlBulkExecute() as sql:
                     for hash, storedValue in shared.inventory.items():
@@ -52,7 +58,7 @@ class singleCleaner(threading.Thread):
                             del shared.inventory[hash]
             shared.UISignalQueue.put(('updateStatusBar', ''))
             shared.broadcastToSendDataQueues((
-                0, 'pong', 'no data'))  # commands the sendData threads to send out a pong message if they haven't sent anything else in the last five minutes. The socket timeout-time is 10 minutes.
+                0, 'pong', 'no data')) # commands the sendData threads to send out a pong message if they haven't sent anything else in the last five minutes. The socket timeout-time is 10 minutes.
             # If we are running as a daemon then we are going to fill up the UI
             # queue which will never be handled by a UI. We should clear it to
             # save memory.
@@ -65,7 +71,7 @@ class singleCleaner(threading.Thread):
                 # inventory (clears pubkeys after 28 days and everything else
                 # after 2 days and 12 hours)
                 sqlExecute(
-                    '''DELETE FROM inventory WHERE (receivedtime<? AND objecttype<>'pubkey') OR (receivedtime<?  AND objecttype='pubkey') ''',
+                    '''DELETE FROM inventory WHERE (receivedtime<? AND objecttype<>'pubkey') OR (receivedtime<? AND objecttype='pubkey') ''',
                     int(time.time()) - shared.lengthOfTimeToLeaveObjectsInInventory,
                     int(time.time()) - shared.lengthOfTimeToHoldOnToAllPubkeys)
 
@@ -75,7 +81,8 @@ class singleCleaner(threading.Thread):
                     int(time.time()) - shared.lengthOfTimeToHoldOnToAllPubkeys)
 
                 queryreturn = sqlQuery(
-                    '''select toaddress, toripe, fromaddress, subject, message, ackdata, lastactiontime, status, pubkeyretrynumber, msgretrynumber FROM sent WHERE ((status='awaitingpubkey' OR status='msgsent') AND folder='sent') ''')  # If the message's folder='trash' then we'll ignore it.
+                    '''select toaddress, toripe, fromaddress, subject, message, ackdata, lastactiontime, status, pubkeyretrynumber, msgretrynumber FROM sent WHERE ((status='awaitingpubkey' OR status='msgsent') AND folder='sent' AND lastactiontime>?) ''',
+                    int(time.time()) - shared.maximumLengthOfTimeToBotherResendingMessages) # If the message's folder='trash' then we'll ignore it.
                 for row in queryreturn:
                     if len(row) < 5:
                         with shared.printLock:
@@ -86,36 +93,12 @@ class singleCleaner(threading.Thread):
                         break
                     toaddress, toripe, fromaddress, subject, message, ackdata, lastactiontime, status, pubkeyretrynumber, msgretrynumber = row
                     if status == 'awaitingpubkey':
-                        if int(time.time()) - lastactiontime > (shared.maximumAgeOfAnObjectThatIAmWillingToAccept * (2 ** (pubkeyretrynumber))):
-                            print 'It has been a long time and we haven\'t heard a response to our getpubkey request. Sending again.'
-                            try:
-                                del shared.neededPubkeys[
-                                    toripe]  # We need to take this entry out of the shared.neededPubkeys structure because the shared.workerQueue checks to see whether the entry is already present and will not do the POW and send the message because it assumes that it has already done it recently.
-                            except:
-                                pass
+                        if (int(time.time()) - lastactiontime) > (shared.maximumAgeOfAnObjectThatIAmWillingToAccept * (2 ** (pubkeyretrynumber))):
+                            resendPubkey(pubkeyretrynumber,toripe)
+                    else: # status == msgsent
+                        if (int(time.time()) - lastactiontime) > (shared.maximumAgeOfAnObjectThatIAmWillingToAccept * (2 ** (msgretrynumber))):
+                            resendMsg(msgretrynumber,ackdata)
 
-                            shared.UISignalQueue.put((
-                                'updateStatusBar', 'Doing work necessary to again attempt to request a public key...'))
-                            t = ()
-                            sqlExecute(
-                                '''UPDATE sent SET lastactiontime=?, pubkeyretrynumber=?, status='msgqueued' WHERE toripe=?''',
-                                int(time.time()),
-                                pubkeyretrynumber + 1,
-                                toripe)
-                            shared.workerQueue.put(('sendmessage', ''))
-                    else:  # status == msgsent
-                        if int(time.time()) - lastactiontime > (shared.maximumAgeOfAnObjectThatIAmWillingToAccept * (2 ** (msgretrynumber))):
-                            print 'It has been a long time and we haven\'t heard an acknowledgement to our msg. Sending again.'
-                            sqlExecute(
-                                '''UPDATE sent SET lastactiontime=?, msgretrynumber=?, status=? WHERE ackdata=?''',
-                                int(time.time()),
-                                msgretrynumber + 1,
-                                'msgqueued',
-                                ackdata)
-                            shared.workerQueue.put(('sendmessage', ''))
-                            shared.UISignalQueue.put((
-                                'updateStatusBar', 'Doing work necessary to again attempt to deliver a message...'))
-                
                 # Let's also clear and reload shared.inventorySets to keep it from
                 # taking up an unnecessary amount of memory.
                 for streamNumber in shared.inventorySets:
@@ -145,3 +128,34 @@ class singleCleaner(threading.Thread):
                 shared.knownNodesLock.release()
                 shared.needToWriteKnownNodesToDisk = False
             time.sleep(300)
+
+
+def resendPubkey(pubkeyretrynumber,toripe):
+    print 'It has been a long time and we haven\'t heard a response to our getpubkey request. Sending again.'
+    try:
+        del shared.neededPubkeys[
+            toripe] # We need to take this entry out of the shared.neededPubkeys structure because the shared.workerQueue checks to see whether the entry is already present and will not do the POW and send the message because it assumes that it has already done it recently.
+    except:
+        pass
+
+    shared.UISignalQueue.put((
+         'updateStatusBar', 'Doing work necessary to again attempt to request a public key...'))
+    t = ()
+    sqlExecute(
+        '''UPDATE sent SET lastactiontime=?, pubkeyretrynumber=?, status='msgqueued' WHERE toripe=?''',
+        int(time.time()),
+        pubkeyretrynumber + 1,
+        toripe)
+    shared.workerQueue.put(('sendmessage', ''))
+
+def resendMsg(msgretrynumber,ackdata):
+    print 'It has been a long time and we haven\'t heard an acknowledgement to our msg. Sending again.'
+    sqlExecute(
+    '''UPDATE sent SET lastactiontime=?, msgretrynumber=?, status=? WHERE ackdata=?''',
+    int(time.time()),
+    msgretrynumber + 1,
+    'msgqueued',
+    ackdata)
+    shared.workerQueue.put(('sendmessage', ''))
+    shared.UISignalQueue.put((
+    'updateStatusBar', 'Doing work necessary to again attempt to deliver a message...'))
