@@ -7,6 +7,8 @@ import sys
 import os
 from debug import logger
 from namecoin import ensureNamecoinOptions
+import random
+import string
 import tr#anslate
 
 # This thread exists because SQLITE3 is so un-threadsafe that we must
@@ -23,6 +25,7 @@ class sqlThread(threading.Thread):
         self.conn = sqlite3.connect(shared.appdata + 'messages.dat')
         self.conn.text_factory = str
         self.cur = self.conn.cursor()
+
         try:
             self.cur.execute(
                 '''CREATE TABLE inbox (msgid blob, toaddress text, fromaddress text, subject text, received text, message text, folder text, encodingtype int, read bool, UNIQUE(msgid) ON CONFLICT REPLACE)''' )
@@ -46,20 +49,18 @@ class sqlThread(threading.Thread):
             # because we may need more flexability in the future and it doesn't
             # take up much more space anyway.
             self.cur.execute(
-                '''CREATE TABLE pubkeys (hash blob, transmitdata blob, time int, usedpersonally text, UNIQUE(hash) ON CONFLICT REPLACE)''' )
+                '''CREATE TABLE pubkeys (hash blob, addressversion int, transmitdata blob, time int, usedpersonally text, UNIQUE(hash, addressversion) ON CONFLICT REPLACE)''' )
             self.cur.execute(
                 '''CREATE TABLE inventory (hash blob, objecttype text, streamnumber int, payload blob, receivedtime integer, tag blob, UNIQUE(hash) ON CONFLICT REPLACE)''' )
-            self.cur.execute(
-                '''CREATE TABLE knownnodes (timelastseen int, stream int, services blob, host blob, port blob, UNIQUE(host, stream, port) ON CONFLICT REPLACE)''' )
-                             # This table isn't used in the program yet but I
-                             # have a feeling that we'll need it.
             self.cur.execute(
                 '''INSERT INTO subscriptions VALUES('Bitmessage new releases/announcements','BM-GtovgYdgs7qXPkoYaRgrLFuFKz1SFpsw',1)''')
             self.cur.execute(
                 '''CREATE TABLE settings (key blob, value blob, UNIQUE(key) ON CONFLICT REPLACE)''' )
-            self.cur.execute( '''INSERT INTO settings VALUES('version','4')''')
+            self.cur.execute( '''INSERT INTO settings VALUES('version','6')''')
             self.cur.execute( '''INSERT INTO settings VALUES('lastvacuumtime',?)''', (
                 int(time.time()),))
+            self.cur.execute(
+                '''CREATE TABLE objectprocessorqueue (objecttype text, data blob, UNIQUE(objecttype, data) ON CONFLICT REPLACE)''' )
             self.conn.commit()
             logger.info('Created messages database file')
         except Exception as err:
@@ -253,13 +254,62 @@ class sqlThread(threading.Thread):
             with open(shared.appdata + 'keys.dat', 'wb') as configfile:
                 shared.config.write(configfile)
 
+        # Add a new column to the pubkeys table to store the address version.
+        # We're going to trash all of our pubkeys and let them be redownloaded.
+        item = '''SELECT value FROM settings WHERE key='version';'''
+        parameters = ''
+        self.cur.execute(item, parameters)
+        currentVersion = int(self.cur.fetchall()[0][0])
+        if currentVersion == 4:
+            self.cur.execute( '''DROP TABLE pubkeys''')
+            self.cur.execute(
+                '''CREATE TABLE pubkeys (hash blob, addressversion int, transmitdata blob, time int, usedpersonally text, UNIQUE(hash, addressversion) ON CONFLICT REPLACE)''' )
+            self.cur.execute(
+                '''delete from inventory where objecttype = 'pubkey';''')
+            item = '''update settings set value=? WHERE key='version';'''
+            parameters = (5,)
+            self.cur.execute(item, parameters)
+            
+        if not shared.config.has_option('bitmessagesettings', 'useidenticons'):
+            shared.config.set('bitmessagesettings', 'useidenticons', 'True')
+        if not shared.config.has_option('bitmessagesettings', 'identiconsuffix'): # acts as a salt
+            shared.config.set('bitmessagesettings', 'identiconsuffix', ''.join(random.choice("123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz") for x in range(12))) # a twelve character pseudo-password to salt the identicons
+            # Since we've added a new config entry, let's write keys.dat to disk.
+            with open(shared.appdata + 'keys.dat', 'wb') as configfile:
+                shared.config.write(configfile)
+
+        #Adjusting time period to stop sending messages
+        if shared.config.getint('bitmessagesettings', 'settingsversion') == 7:
+            shared.config.set(
+                'bitmessagesettings', 'stopresendingafterxdays', '')
+            shared.config.set(
+                'bitmessagesettings', 'stopresendingafterxmonths', '')
+            #shared.config.set(
+            shared.config.set('bitmessagesettings', 'settingsversion', '8') 
+            with open(shared.appdata + 'keys.dat', 'wb') as configfile:
+                shared.config.write(configfile)
+
+        # Add a new table: objectprocessorqueue with which to hold objects
+        # that have yet to be processed if the user shuts down Bitmessage.
+        item = '''SELECT value FROM settings WHERE key='version';'''
+        parameters = ''
+        self.cur.execute(item, parameters)
+        currentVersion = int(self.cur.fetchall()[0][0])
+        if currentVersion == 5:
+            self.cur.execute( '''DROP TABLE knownnodes''')
+            self.cur.execute(
+                '''CREATE TABLE objectprocessorqueue (objecttype text, data blob, UNIQUE(objecttype, data) ON CONFLICT REPLACE)''' )
+            item = '''update settings set value=? WHERE key='version';'''
+            parameters = (6,)
+            self.cur.execute(item, parameters)
+
         # Are you hoping to add a new option to the keys.dat file of existing
         # Bitmessage users? Add it right above this line!
         
         try:
             testpayload = '\x00\x00'
-            t = ('1234', testpayload, '12345678', 'no')
-            self.cur.execute( '''INSERT INTO pubkeys VALUES(?,?,?,?)''', t)
+            t = ('1234', 1, testpayload, '12345678', 'no')
+            self.cur.execute( '''INSERT INTO pubkeys VALUES(?,?,?,?,?)''', t)
             self.conn.commit()
             self.cur.execute(
                 '''SELECT transmitdata FROM pubkeys WHERE hash='1234' ''')
