@@ -3,6 +3,10 @@ import ConfigParser
 import sys
 import os
 import locale
+import random
+import string
+import platform
+from distutils.version import StrictVersion
 
 from namecoin import ensureNamecoinOptions
 
@@ -42,7 +46,7 @@ def loadConfig():
         # This appears to be the first time running the program; there is
         # no config file (or it cannot be accessed). Create config file.
         shared.config.add_section('bitmessagesettings')
-        shared.config.set('bitmessagesettings', 'settingsversion', '7')
+        shared.config.set('bitmessagesettings', 'settingsversion', '8')
         shared.config.set('bitmessagesettings', 'port', '8444')
         shared.config.set(
             'bitmessagesettings', 'timeformat', '%%a, %%d %%b %%Y  %%I:%%M %%p')
@@ -84,6 +88,17 @@ def loadConfig():
             'bitmessagesettings', 'maxacceptablepayloadlengthextrabytes', '0')
         shared.config.set('bitmessagesettings', 'dontconnect', 'true')
         shared.config.set('bitmessagesettings', 'userlocale', 'system')
+        shared.config.set('bitmessagesettings', 'useidenticons', 'True')
+        shared.config.set('bitmessagesettings', 'identiconsuffix', ''.join(random.choice("123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz") for x in range(12))) # a twelve character pseudo-password to salt the identicons
+        
+         #start:UI setting to stop trying to send messages after X days/months
+        shared.config.set(
+            'bitmessagesettings', 'stopresendingafterxdays', '')
+        shared.config.set(
+            'bitmessagesettings', 'stopresendingafterxmonths', '')
+        #shared.config.set(
+        #    'bitmessagesettings', 'timeperiod', '-1')
+        #end
 
         # Are you hoping to add a new option to the keys.dat file? You're in
         # the right place for adding it to users who install the software for
@@ -106,3 +121,13 @@ def loadConfig():
             os.umask(0o077)
         with open(shared.appdata + 'keys.dat', 'wb') as configfile:
             shared.config.write(configfile)
+
+def isOurOperatingSystemLimitedToHavingVeryFewHalfOpenConnections():
+    try:
+        VER_THIS=StrictVersion(platform.version())
+        if sys.platform[0:3]=="win":
+            return StrictVersion("5.1.2600")<=VER_THIS and StrictVersion("6.0.6000")>=VER_THIS
+        return False
+    except Exception as err:
+        print 'An Exception occurred within isOurOperatingSystemLimitedToHavingVeryFewHalfOpenConnections:', err
+        return False
\ No newline at end of file
